'use strict';
var yeoman = require('yeoman-generator');
var chalk = require('chalk');
var yosay = require('yosay');
var path = require('path');

var purecloudcsharpgenerator = yeoman.generators.Base.extend({
    constructor: function() {
        yeoman.generators.Base.apply(this, arguments);
    },

    init: function() {
        // Have Yeoman greet the user.
        this.log(yosay('Welcome to the marvellous ' + chalk.red('PureCloud C# App') + ' generator!'));
        this.templatedata = {};
    },
    getInfo: function() {
        var done = this.async();

        function askForName() {
            var prompts = [{
                name: 'applicationName',
                message: 'What\'s the name of your application?',
                default: 'PureCloudCSharpApp'
            }];
            this.prompt(prompts, function(props) {
                this.templatedata.namespace = props.applicationName;
                this.templatedata.applicationName = props.applicationName;
                this.applicationName = props.applicationName;
                askForEnvironment.apply(this);
            }.bind(this));
        }

        function askForEnvironment() {
            var prompts = [{
                type: 'list',
                name: 'environment',
                message: 'What\'s your PureCloud Environment?',
                choices: [{
                        name: 'mypurecloud.com',
                        value: 'mypurecloud.com'
                    },
                    {
                        name: "mypurecloud.ie",
                        value: 'mypurecloud.ie'
                    },
                    {
                        name: 'mypurecloud.com.au',
                        value: 'mypurecloud.com.au'
                    },
                    {
                        name: 'mypurecloud.jp',
                        value: 'mypurecloud.jp'
                    }
                ]
            }];

            this.prompt(prompts, function(props) {
                this.templatedata.environment = props.environment;
                askForClientId.apply(this);
            }.bind(this));
        }

        function askForClientId() {
            var prompts = [{
                name: 'clientId',
                message: 'What\'s your client id (Implicit Grant)?',
                default: 'xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx'
            }];
            this.prompt(prompts, function(props) {
                this.templatedata.clientId = props.clientId;
                askForRedirectUri.apply(this);
            }.bind(this));
        }

        function askForRedirectUri() {
            var prompts = [{
                name: 'redirectUri',
                message: 'What\'s your redirect uri?',
                default: 'http://localhost:8080/index.html'
            }];
            this.prompt(prompts, function(props) {
                this.templatedata.redirectUri = props.redirectUri;
                done();
            }.bind(this));
        }

        askForName.apply(this);
    },

    writing: function() {
        this.mkdir(this.applicationName);

        this.sourceRoot(path.join(__dirname, './templates/'));

        // CS Files
        this.template(this.sourceRoot() + '/Program.cs', this.applicationName + '/Program.cs', this.templatedata);
        this.template(this.sourceRoot() + '/Form1.cs', this.applicationName + '/Form1.cs', this.templatedata);
        this.template(this.sourceRoot() + '/Form1.Designer.cs', this.applicationName + '/Form1.Designer.cs', this.templatedata);
        this.template(this.sourceRoot() + '/Form1.resx', this.applicationName + '/Form1.resx', this.templatedata);

        // Configuration Files
        this.template(this.sourceRoot() + '/App.config', this.applicationName + '/App.config', this.templatedata);
        this.template(this.sourceRoot() + '/packages.config', this.applicationName + '/packages.config', this.templatedata);

        // Project & Solution Files
        this.template(this.sourceRoot() + '/NetCallControls.csproj', this.applicationName + '/' + this.applicationName + '.csproj', this.templatedata);
        this.template(this.sourceRoot() + '/NetCallControls.sln', this.applicationName + '/' + this.applicationName + '.sln', this.templatedata);

        // Properties
        this.mkdir(this.applicationName + '/Properties/');
        this.template(this.sourceRoot() + '/Properties/AssemblyInfo.cs', this.applicationName + '/Properties/AssemblyInfo.cs', this.templatedata);
        this.template(this.sourceRoot() + '/Properties/Resources.resx', this.applicationName + '/Properties/Resources.resx', this.templatedata);
        this.template(this.sourceRoot() + '/Properties/Resources.Designer.cs', this.applicationName + '/Properties/Resources.Designer.cs', this.templatedata);
        this.template(this.sourceRoot() + '/Properties/Settings.settings', this.applicationName + '/Properties/Settings.settings', this.templatedata);
        this.template(this.sourceRoot() + '/Properties/Settings.Designer.cs', this.applicationName + '/Properties/Settings.Designer.cs', this.templatedata);
    }
});

module.exports = purecloudcsharpgenerator